/* $Id: reset.css 644946cbfc5c 2012/03/14 18:21:43 patrick $ */

/* Reset */
body,div,dl,dt,dd,ul,ol,li,h1,h2,h3,h4,h5,h6,pre,code,form,fieldset,legend,input,textarea,p,blockquote,th,td { margin:0; padding:0; }
table { border-collapse:collapse; border-spacing:0; }
fieldset,img { border:0; }
address,caption,cite,code,dfn,em,strong,th,var { font-style:normal; font-weight:normal; }
li { list-style:none; }
caption,th { text-align:left; }
h1,h2,h3,h4,h5,h6 { font-size:100%; font-weight:normal; }
q:before,q:after { content:''; }
abbr,acronym { border:0; font-variant:normal; }
input,textarea,select { font-family:inherit; font-size:inherit; font-weight:inherit; }
legend { color:#000; }
* html input,* html textarea,* html select { font-size:100%; }

/* Fonts */
body { font:13px/1.231 Helvetica, 'Liberation Sans', FreeSans, sans-serif; }
* html body { font-size:small; }
select,input,button,textarea { font:99% Helvetica, 'Liberation Sans', FreeSans, sans-serif; }
table { font-size:inherit; font-size:100%; }
pre,code,kbd,samp,tt { font-family:monospace; line-height:100%; }
* html pre,* html code,* html kbd,* html samp,* html tt { font-size:108%; }

/* Presentation */
h1 { font-size:146.5%; }
h2 { font-size:123.1%; }
h3 { font-size:108%; }
h1,h2,h3 { margin:1em 0; }
h1,h2,h3,h4,h5,h6,strong { font-weight:bold; }
abbr,acronym { border-bottom:1px dotted #000; cursor:help; }
em { font-style:italic; }
blockquote,ul,ol,dl { margin:1em; }
ol,ul,dl { margin-left:2em; }
ol li { list-style: decimal outside; }
ul li { list-style: disc outside; }
dl dd { margin-left:1em; }
th,td { border:1px solid #000; padding:.3em; }
th { font-weight:bold; text-align:center; }
caption { margin-bottom:.5em; text-align:center; }
p,fieldset,table,pre { margin-bottom:1em; }
fieldset { overflow:hidden; }
input[type=text],input[type=password],textarea{ width:12.25em; }
* html input[type=text],* html input[type=password],* html textarea { width:11.9em; }

/* Layout */
#mz { text-align: center; }
#mzfull,#mz780,#mz975 { margin:auto; min-width:40em; text-align:left; }
#mz780 { width:60em; }
* html #mz780 { width:58.5em; }
#mz975 { width:75em; }
* html #mz975 { width:73.125em; }
.colFirst { margin:0; padding-right:1%; float:left; }
.col { margin:0; padding:0 1% 0 1%; float:left; }
.colLast { margin:0; padding-left:1%; }
.col11 .colFirst { width:49%; }
.col11 .colLast { margin-left:50%; }
.col21 .colFirst { width:66%; }
.col21 .colLast { margin-left:67%; }
.col31 .colFirst { width:75%; }
.col31 .colLast { margin-left:76%; }
.col12 .colFirst { width:33%; }
.col12 .colLast { margin-left:34%; }
.col111 .colFirst, .col111 .col { width:32%; }
.col111 .colLast { margin-left:67%; }
.col13 .colFirst { width:24%; }
.col13 .colLast { margin-left:25%; }
.col1111 .colFirst, .col1111 .col { width:24%; }
.col1111 .colLast { margin-left:76%; }

/* Miscellaneous */
.clear { clear:both; display:block; overflow:hidden; visibility:hidden; width:1px; height:1px; }
.hidden { display:none; }