/* $Id: publidoc.css e94b9008414b 2012/03/15 12:14:15 patrick $ */


/* ======= TOP LEVEL ====== */


/* ~~~~~~~ pdocCover ~~~~~~ */

body.pdocCover {
    margin: 0;
    padding: 0;
    text-align: center;
}

body.pdocCover img { height: 100%; }

/* ~~~~~~~~ pdocToc ~~~~~~~ */

.pdocToc ul li { font-weight: bold; font-size: large; }
.pdocToc ul ul li { font-weight: bold; font-size: medium; }
.pdocToc ul a { font-weight: normal; font-size: small; }

/* ~~~~~ pdocAbstract ~~~~~ */

.pdocAbstract {
    font-family: sans-serif;
    font-size: small;
    font-weight: normal;
}

/* ~~~~~~~~ pdocNav ~~~~~~~ */

.pdocNav .col { text-align: center; }
.pdocNav .colLast { text-align: right; }
.pdocNav hr { clear: both; }

/* ~~~~~~~ pdocTopic ~~~~~~ */

body.pdocTopic {
    padding: 0 1em 0 1em;
}

div.pdocTopic {
    padding: .8em 0 .8em 0;
    border-bottom: 1px dotted #000000;
}

body.pdocTopic-title {
    text-align: center;
}

body.pdocTopic-title h1,
body.pdocTopic-title .h1 {
    margin: 3em 0 2em 0;
    text-align: center;
}

body.pdocTopic-title h2,
body.pdocTopic-title .h2 {
    margin-bottom: 10em;
    text-align: center;
}

body.pdocTopic-title .pdocP {
    text-align: center;
}

body.pdocTopic-copyright {
    padding: 1em;
    text-align: left;
}

body.pdocTopic-copyright .pdocSection1 {
    margin-bottom: 1em;
}

body.pdocTopic-copyright .pdocP {
    margin-bottom: 0;
}


/* ======= META LEVEL ======= */


/* ~~~~~~~~ pdocMeta ~~~~~~~~ */

.pdocMeta {
    margin: 1.33ex 0;
    border: thin solid #c0f8f8;
    background-color: #e0f8f8;
    padding: 0.5em;
}


/* ===== SECTION LEVEL ====== */


/* ~~~~~~~ pdocSection ~~~~~~ */

.pdocSection1, .pdocSection2, .pdocSection3 {
    font-family: serif;
    text-align: justify;
    margin-bottom: .8em;
}

.pdocSectionTitle {
    font-family: sans-serif;
    font-weight: bold;
    margin: .5em 0 .5em 0;
    font-size: 105%;
}

.pdocSectionSubtitle {
    font-family: sans-serif;
    font-weight: bold;
    margin: .3em 0 .3em 0;
}

.pdocSection-sign {
    text-align: right;
    font-weight: bold;
}

.pdocSection-box {
    margin: 1em 0 1em 0;
    padding: .5em 1em .5em 1em;
    border: 1px solid #000000;
    background-color: #efefef;
}
.pdocSection-box .pdocSectionTitle,
.pdocSection-box .pdocSectionSubtitle {
    text-align: center;
}
    
.pdocSection-logo {
    text-align: center;
}

.pdocSection-ex {
    font-style: italic;
}

/* ====== BLOCK LEVEL ======= */


/* ~~~~~~~~~~ pdocP ~~~~~~~~~ */

.pdocSection1 .pdocP,
.pdocSection2 .pdocP,
.pdocSection3 .pdocP {
    display: block;
    margin-bottom: .4em;
}

/* ~~~~~~~ pdocSpeech ~~~~~~~ */

.pdocSpeechSpeaker {
  font-family: sans-serif;
}

.pdocSpeechStage {
  font-family: sans-serif;
}

/* ~~~~~~~~ pdocList ~~~~~~~~ */

.pdocListTitle {
    font-family: sans-serif;
    font-weight: bold;
}

.pdocListGlossary {
    margin-left: 0;
}

.pdocListGlossary li { 
    list-style-type: none;  
}

/* ~~~~~~~~ pdocQuote ~~~~~~~ */

.pdocQuote {
    font-style: italic;
}

/* ~~~~~~~ pdocTable ~~~~~~~~ */

.pdocTable {
    width: 100%;
}

.pdocTableTitle {
    font-family: sans-serif;
    font-weight: bold;
    margin-bottom: .3em;
}

.pdocTableCaption {
    color: #004080;
    font-size: 85%;
}

.pdocTable td {
    vertical-align: top;
}

/* ~~~~~~~ pdocMedia ~~~~~~~~ */

.pdocMediaTitle {
    font-family: sans-serif;
    font-weight: bold;
}

.pdocMedia {
    margin: .8em 0 .8em 0;
    font-family: sans-serif;
}

.pdocMediaCopyright {
    color: gray;
    font-style: italic;
    font-size: 75%;
}

.pdocMediaCaption {
    color: #004080;
    font-size: 85%;
}


/* ====== INLINE LEVEL ====== */


/* ~~~~~~~~ pdocNote ~~~~~~~~ */

.pdocNoteCall {
    color: #993333;
    text-decoration: none;
    font-weight: bold;
    cursor: help;
}

.pdocNoteFooter {
    clear: both;

    text-align: left;
    margin-top: 1em;
    padding-top: .5em;
    font-size: 85%;
    border-top: 1px solid #000000;
}
.pdocNoteFooter a  {
    color: #993333;
    text-decoration: none;
    font-weight: bold;
}

/* ~~~~~~ Miscellaneous ~~~~~ */

sup { font-size: 70%; }
sub { font-size: 70%; top: -1.8em; }

.pdocMathWide { display: block; text-align: center; }

.pdocNumberRoman { font-variant: small-caps; }

.pdocAcronym {
    font-variant: small-caps;
    font-size: 110%;
}

.pdocLink {
    font-family: serif;
}