/* $Id: main.css d3a05f3a3f66 2012/01/22 11:07:13 patrick $ */
/* Publidoc2Html */

body {
  margin: 1em;
  font-family: sans-serif;
}

h1 { font-size:180%; font-variant: small-caps; }
h2 { font-size:123.1%; }
h3 { font-size:108%; }
h4 { margin: .8em 0; color: #993333; border-bottom: 1px solid #993333; }

.h1 { margin:1em 0; font-size:180%; font-weight: bold; font-variant: small-caps; }
.h2 { margin:1em 0; font-size:123.1%; font-weight: bold; }
.h3 { margin:1em 0; font-size:108%;   font-weight: bold; }
.h4 { margin: .8em 0; color: #993333; border-bottom: 1px solid #993333; }
