/* $Id: base.css 7af639d5ec56 2012/03/18 16:06:23 patrick $ */

/* ======= Presentation ===== */

body { background-color: #4f4f4f; }
h1 { color: #993333; }
h2 { color: #4f4f4f; text-shadow: 1px 1px 2px #bbbbbb; }
h3 { color: #c05800; text-shadow: 1px 1px 2px #bbbbbb; font-family: sans-serif; }
a { text-decoration: none; color: black; }
a:visited { text-decoration: none; color: black; }
a:hover { text-decoration: underline; }
h3 a:link, h3 a:visited { color: #c05800; }
input[type="text"], input[type="password"], input[type="checkbox"],
input[type="file"], select, textarea {
  border:1px solid #bbbbbb;
  background-color: #ffffff;
}

/* ========== Header ======== */

#header { background-color: #fffff0; }

#header h1 { float:left; font-size: 197%; margin: 10px; min-height: 30px; }
#header h1 span { color: #993333; }
#header .clear { visibility: visible; width: 100%; background-color: black; }

#header h1 a {
  position: absolute;
  left: 10px;
  top: 10px;
  width: 149px;
  height: 37px;
  cursor: pointer;
  background: transparent url('../Images/publiforge.png') no-repeat top left;
}

#user { padding-top: 2.6em; }

#userPre {
  display: block;
  float: right;
  height: 1.4em;
  width: 2em;
  padding: .3em 0 .1em 0;
  background: transparent url('../Images/user_pre.png') no-repeat top right;
}

#userName  {
  display :block;
  float: right;
  height: 1.4em;
  padding: .3em .5em .1em .5em;
  background-color: #ddddcc;
}
#userName a { font-size: 85%; }

#userTools {
  display: block;
  float: right;
  height: 1.4em;
  padding: .3em .5em .1em .5em;
  background-color: #4f4f4f;
}
#userTools img { vertical-align: middle; }

* html #userPre, * html #userName, * html #userTools { height: 1.6em; padding-top: .4em; }

/* ======== Breadcrumbs ===== */

#breadcrumbs {
  padding: 12px 10px 8px 10px;
  color: white;
  background: #993333 url('../Images/breadcrumbs_top.png') repeat-x left top;
  border-bottom: 1px solid #000000;
}

#breadcrumbs .first {
  float:right;
  margin-left: 1em;
  text-align: right;
  font-variant: small-caps;
}
#breadcrumbs .first h1 {
  margin: 0;
  color: white;
  font-size: 85%;
  font-weight: normal;
}

#breadcrumbs .last { font-size: 85%; }
#breadcrumbs .last a { color: white; }

/* ========= Content ======== */

#content1 {
  position: relative;
  width: 100%;
  overflow: hidden;
  background-color: white;
  min-width: 55em;
}

#content2 {
  float: left;
  position: relative;
  width: 100%;
  right: 78%;
  background-color: #ddddcc;
  border-right: 1px solid #000000;
}

/* =========== Left ========= */

#leftPanel {
  float: left;
  position: relative;
  overflow: hidden;
  width: 22%;
  left: 78%;
}

#leftSlide { padding-bottom: 1em; }

#leftClose {
  text-align: right;
  padding-right: 5px;
  background: #c7c7b7 url('../Images/left_title.png') repeat-x top left;
  border-bottom: 1px solid #aeaea0;
}
#leftClose a {
  font-size: 120%;
  font-weight: bold;
  text-decoration: none;
  outline: none;
}

.leftWidget { margin: 0; }

.leftWidget legend {
  width: 100%;  
  padding: .4em 5px .4em 5px;
  color: #993333;
  font-weight: bold;
  background: #c7c7b7 url('../Images/left_title.png') repeat-x top left;
  border-top: 1px solid #fbfbe8;
  border-bottom: 1px solid #aeaea0;
}

/* =========== Main ========= */

#mainPanel {
  float: left;
  position: relative;
  overflow: hidden;
  width: 78%;
  left: 78.09%;
}
* html #mainPanel { width: 77.99%; }
*+html #mainPanel { width: 77.99%; }

#main { margin: 10px; }

/* ========= Subtitle ======= */

#subtitle {
  height: 32px;
  margin: 0;
  padding-right: 40px;
  text-align: right;
}

#subtitle.user {
  background: transparent url('../Images/user32.png') no-repeat top right;
}
#subtitle.group {
  background: transparent url('../Images/group32.png') no-repeat top right;
}
#subtitle.storage {
  background: transparent url('../Images/storage32.png') no-repeat top right;
}
#subtitle.search {
  background: transparent url('../Images/search32.png') no-repeat top right;
}
#subtitle.file {
  background: transparent url('../Images/file32.png') no-repeat top right;
}
#subtitle.folder {
  background: transparent url('../Images/folder32.png') no-repeat top right;
}
#subtitle.project {
  background: transparent url('../Images/project32.png') no-repeat top right;
}
#subtitle.processing {
  background: transparent url('../Images/processing32.png') no-repeat top right;
}
#subtitle.pack {
  background: transparent url('../Images/pack32.png') no-repeat top right;
}
#subtitle.build {
  background: transparent url('../Images/build32.png') no-repeat top right;
}

#subtitle em {
  display: block;
  margin-top: -.2em;
  font-style: normal;
  font-size: 69%;
  text-shadow: none;
}

/* ========== Footer ======== */

#footer {
  padding: 14px 10px 10px 10px;
  color: #999999;
  text-shadow: 0 -1px 0 #000000;
  border-top: 1px solid #000000;
  background: #4f4f4f url('../Images/footer_top.png') repeat-x top left;
}

#footer a { color: #999999; }
#footer img { vertical-align: bottom; }
#footer .colFirst { text-align: left; }
#footer .col { text-align: center; }
#footer .colLast { text-align: right; }

/* =========== Menu ========= */

#menu legend { display: none; }

#menu ul { margin: 0; }
#menu ul li { list-style-type: none; list-style-position: inside; }
#menu ul li strong {
  display: block;  
  padding: .4em 5px .4em 5px;
  color: #993333;
  background: #c7c7b7 url('../Images/left_title.png') repeat-x top left;
  border-top: 1px solid #fbfbe8;
  border-bottom: 1px solid #aeaea0;
}

#menu ul ul { margin: 5px 0 5px 0; }
#menu ul ul li {
  margin-bottom: .4em;
  padding: 0 5px 0 5px;
  color: #4f4f4f;
  font-weight: bold;
  text-shadow: 1px 1px 1px #aeaea0;
}
#menu ul ul li em { font-style: normal; }
#menu ul ul li a { display: block; color: #4f4f4f; }
#menu ul ul li a:hover { text-decoration: none; background-color: #c7c7b7; }
#menu ul ul li ul { display: none; }
#menu ul ul li.active ul { display: block; }

#menu ul ul ul { margin: .1em 0 0 1.3em; font-size: 89%; }
#menu ul ul ul li { font-weight: normal; margin-bottom: .1em; }

#menu a.current { color: #3333ff; }

#menu img { vertical-align: middle; }

/* ========= Buttons ======== */

#buttons {
  padding: .4em 10px 0 10px;
  height: 2em;
  color: white;
  background: #4f4f4f;
}

#buttons .first { float: left;  padding-top: .1em; }
#buttons .last { text-align: right; }
#buttons img,
#buttons input[type="image"] { vertical-align: middle; }
    
.button, .buttonMain {
  cursor: pointer;
  padding: .1em .4em .1em .4em;
  border: 1px solid #999999;
  color: #004080;
  background: #cccccc url('../Images/button.png') repeat-x left top;
  text-decoration: none;
  font-size: 93%;
  font-weight: normal;
}
a.button, a.buttonMain, a.button:visited, a.buttonMain:visited {
  padding: .15em .45em .15em .45em;
  color: #004080;
}
a.button img { vertical-align: middle; }
a.button:hover { text-decoration: none; }
a.button:hover, input.button:hover {
  background: #dddddd url('../Images/button_over.png') repeat-x left top;
}

.buttonMain { font-weight: bold; }

/* =========== Form ========= */

.formItem label em { color: #004080; font-weight: bold; font-style: normal; }

.formItem img,
.formItem input[type="image"] { vertical-align: middle; }
.formItem input[type="text"],
.formItem input[type="password"] { width: 50%; margin-bottom: .2em;}
.formItem input[type="text"].picker { width: 6em; }
.formItem input[type="checkbox"] { margin-top: 3px; }
.formItem ul { margin: .5em 0 0 0; font-family: sans-serif; }
.formItem ul ul { margin: 0 0 0 1.5em; }
.formItem ul li {
  list-style-type: none;
  list-style-position: inside;
}
.formItem div em { font-family: sans-serif; font-style: normal; }
.formItem hr { margin: 1.2em 0 1em 0; }
.formItem h3 { margin: .6em 0 .2em 0; }

.formCheck { margin-bottom: .3em; }
* html .formCheck { margin-bottom: .29em; }
.formCheck label { color: #004080; font-weight: bold; }

/* ======= Description ====== */

.description {
  margin-bottom: 1em;
}

.description div { text-align: justify; }

#descriptionProject {
  margin: .5em 0 1.5em 0;
  padding: 5px;
  background-color: #ffffdd;
  border: 1px dashed #bbbbbb;
  box-shadow: 2px 2px 3px #bbbbbb;
}

#descriptionProject .description {
  font-size: 116%;
  margin: 0;
}

#descriptionProject .formItem {
  margin: .4em 0 0 0;
}
#descriptionProject .formItem label {
  width: 8%;
}
#descriptionProject .formItem div {
  margin-left: 9%;
}

/* ========== Filter ======== */

.filter h3 {
  display: inline;
  padding: 4px 0 2px 5px;
  color: #004080;
  font-size: 85%;
  background: white url('../Images/tab.png') left -80px;
}
.filter h3 span {
  padding: 4px 12px 1px 0;
  background: white url('../Images/tab.png') right -80px; 
}
* html .filter h3 { padding-top: 0; }
*+html .filter h3 { padding-top: 0; }

.filter .col11 {
  padding: 8px 5px 5px 5px;
  border: #999999 solid 1px;
  background-color: #fafafa;
}
.filter .col11 .colFirst { font-size: 85%; }
.filter .col11 .colLast { font-size: 85%; }
* html .filter .col11 .colFirst { position: relative; }
* html .filter .col11 .colLast label { position: relative; }

.filter input[type="text"] { width: 90%; }
.filter .submit { clear: both; margin-top: .5em; font-size: 93%; }
.filter .clear { clear: right; }

/* ======== toolTip ========= */

#toolTip {
  padding: 10px;
  margin: .5em 0 1.1em 0;
  background-color: #e8f0f7;
  border: 1px dashed #5f7ab1;
  box-shadow: 2px 2px 3px #bbbbbb;
}

/* ======== dialogBox ======= */

#dialogBox {
  margin: .5em 0 .5em 0;
  background-color: #e8f0f7;
  border: 1px solid #5f7ab1;
  box-shadow: 2px 2px 3px #bbbbbb;
}

#dialogBox .legend {
  padding: 2px 5px 2px 5px;
  font-weight: bold;
  color: white;
  background-color: #5f7ab1;
}
#dialogBox .legend .first { float:left; }
#dialogBox .legend .last { text-align: right; }

#dialogBox .content { margin: 5px; }

#dialogBox .listActions { background-color: #fafafa; }
#dialogBox .listPager { background-color: #fafafa; }

/* ======== Container ======= */

#container {
  margin: .5em 0 .5em 0;
  border: 1px solid #c05800;
  box-shadow: 2px 2px 3px #bbbbbb;
}

#container h3 { display: none; }

#container .legend {
  padding: 2px 5px 2px 5px;
  font-size: 85%;
  font-weight: bold;
  color: white;
  background-color: #c05800;
}
#container .legend .first { float:left; }
#container .legend .last { text-align: right; }

#container .content { margin: 5px; font-weight: bold; color: #4f4f4f; }
#container .content a { font-size: 93%; color: #4f4f4f; }

#container .col111 { margin: 2px 0 0 0; text-align: center; }
#container .colFirst div, #container .col div, #container .colLast div {
  color: black;
  font-weight: normal;
  font-size: 93%;
  padding: 4px;
  border: 1px solid #c05800;
  box-shadow: 1px 1px 2px #bbbbbb;
}

#container input, #container img { vertical-align: middle; }

#container .nav { margin-bottom: .5em; }
#container .nav .first { float:left; margin-bottom: 5px; }
#container .nav .last { text-align: right; font-weight: normal; font-size: 93%; }
#container .flash { color: #339933; }

/* ====== List - browse ===== */

h2.list { margin: 0 0 .5em 0; background-color: transparent; }
h3.list { display: none; }

/* listActions */
.listActions {
  margin-top: .4em;
  padding: 0 5px .5em 5px;
  background-color: #ffffee;
  border-top: #999999 dotted 1px;
  text-shadow: 1px 1px 1px #bbbbbb;
}
.listActions .first {
  float: left;
  margin: .3em 1em 0 0;
  font-size: 108%;
  color: #5f7ab1;
  font-weight: bold;
}
.listActions .last {
  text-align: right;
  margin: .4em 0 0 0;
}
.listActions h2 { margin: 0; }
.listActions span { color: #3333ff; }
.listActions .path h3 { margin: 0; color: black; text-shadow: none; }
.listActions .path a { color: black; }
.listActions .path span { color: #3333ff; }
.listActions .processing { text-shadow: none; color: #7f7f7f; }
.listActions img, .listActions input { vertical-align: middle; }
* html .listActions .first { position: relative; }

/* list - browse */
.list {
  width: 100%;
  margin: 0;
  background-color: white;
  font-family: sans-serif;
}

.list caption { display: none; }

.list tr { border-top: #999999 dashed 1px; vertical-align: top; }
.list tr#current { background-color: #e8f0f7; }
.list tr:hover { background-color: #e8f0f7; }
.browse tr { border-top: none; }

.list th {
  padding: .4em .3em .3em .3em;
  text-align: left;
  vertical-align: middle;
  background: #dfded0 url('../Images/list_header.png') repeat-x left top;
  border-top: #999999 solid 1px;
  border-right: none;
  border-bottom: #999999 solid 1px;
  border-left: none;
  text-shadow: 1px 1px 1px #999999;
}
.list th:first-child { padding-left: 5px; }
.list th.last { text-align: right; padding-right: 5px; }
.list th.checkbox { width: 1em; padding-left: 7px; text-align: left; }
.list th.center { text-align: center; }
.group th { vertical-align: top; background: none; color: #7f7f7f; text-shadow: none; }
.browse th { text-align: right; }
.browse th.first { text-align: left; }
.browse th.last { text-align: left; }

.list td { border: none; font-size: 93%; }
.list td:first-child { padding-left: 5px; }
.list td.last { text-align: right; padding-right: 5px; }
.list td.checkbox { width: 1em; padding-left: 7px; text-align: left; }
.list td.info { color: #7f7f7f; font-size: 85%; }
.list td.center { text-align: center; }
.list td.actions { white-space: nowrap; }
.list td form { font-size: 85%; color: #3333ff; }
.list td div { margin: 0; }
.list td input[type="text"]  { width: 100%; }
.list td input[type="checkbox"]  { margin: 0; }
.browse td { padding: .2em .5em .2em .5em; text-align: right; white-space: nowrap; }
.browse td.first { text-align: left; }
.browse td.last { text-align: left; }

.list img { vertical-align: middle; }
.list input[type="image"] { vertical-align: middle; }
.list select { font-size: 85%; }

.list a:focus { border: none; }
.browse a:hover { text-decoration: none; }

/* listPager */
.listPager {
  padding: .3em 5px .3em 5px;
  border-top: #999999 solid 1px;
  border-bottom: #999999 dotted 1px;
  background-color: #ffffee;
  font-weight: bold;
  text-shadow: 1px 1px 1px #bbbbbb;
}
.listPager .first {
  float: left;
  color: #3333ff;
  font-size: 108%;
}
.listPager .last {
  font-family: sans-serif;
  text-align: right;
  color: #7f7f7f;
  font-size: 85%;
  text-shadow: none;
  font-weight: normal;
}
.listPager img { vertical-align: middle; }
.listPager a { color: #5f7ab1; }

/* smallList */
.smallList .listActions {
  margin: 0;
  color: #7f7f7f;
  background-color: transparent;
  border-top: none;
  font-size: 93%;
}
.smallList .listActions span { color: black; }
.smallList tr { border-top: #999999 dotted 1px; }
.smallList th { font-size: 85%; text-shadow: none; }
.smallList td { font-size: 85%; text-shadow: none; padding-bottom: .1em; }
.smallList.listPager { background-color: #fafafa; padding-top: .1em; }

/* browseMsg */
#browseMsg {
  padding: 1em;
  margin: 1em 0 1em 0;
  font-size: 108%;
  font-weight: bold;
  color: #993333;
  background-color: #ffeeee;
  border: 1px dotted #993333;
}

/* =========== TabSet ========= */

#tabSet { margin: 1em 0 1em 0; }

#tabSet .listActions { background-color: #e8f0f7; }

.tabToc {
  height: 2em;
  margin: 0;
  padding: 0;
  font-size:85%;
  border-bottom: #999999 solid 1px;
}

.tabToc li {
  float: left;
  list-style-type: none;
  margin-left: -1px;
  background-color: #e8f0f7;
}
.tabToc li:first-child { margin-left: 0; }
* html .tabToc li { margin-left: 0; }

.tabToc a {
  float: left;
  display: block;
  padding-left: 10px;
  color: #5f7ab1;
  background: white url('../Images/tab.png');
  font-weight: bold;
  text-decoration: none;
  outline: none;
}
.tabToc a:visited { color: #5f7ab1; }
.tabToc a:hover {
  color: #5f7ab1;
  background-color: #fafafa;
  background-position: left -80px;
}
.tabToc a:hover span { background-position: right -80px; }
* html a { padding-bottom: 1px; }

.tabToc li.tabCurrent { background-color: #fafafa; }
.tabToc li.tabCurrent a {
  background-position: left -80px;
  padding-bottom: 1px;
}
.tabToc li.tabCurrent span { background-position: right -80px; }

.tabToc span {
  display: block;
  line-height: 2em;
  padding-right: 14px;
  background: white url('../Images/tab.png') right top;
}

.tabContent {
  clear: left;
  background-color: #fafafa;
  border-top: none;
  border-right: #999999 solid 1px;
  border-bottom: #999999 solid 1px;
  border-left: #999999 solid 1px;
  padding: 10px;
}
* html .tabContent { border-top-color: #fafafa; }
*+html .tabContent { border-top-color: #fafafa; }
.tabContent legend { margin:0; padding:0; }
.tabContent legend span { display:none; }
.tabContent div.info { margin-bottom: .5em; }
.tabContent ul.item { margin: 0 1em 0 0; }
.tabContent ul.checkbox { margin: 0 1em 0 0; }
.tabContent ul.checkbox li { list-style-type: none; }
.tabContent .menu_items { margin-bottom: .8em; }
.tabContent .menu_items ul li {
  list-style-type: square;
  list-style-position: inside;
}
.tabContent .menu_items ul ul { margin-top: .3em; }
.tabContent #description { width: 100%; }
.tabContent .date input { width: 7em; }
.tabContent input[name="new_ip"] { width: 8em; }

/* ============ Work ======== */

#work {
  margin: 2em 0 2em 0;  
}

#work .col { text-align: center; }
#work .colLast { text-align: right; }

#work .button {
  padding: .5em 1em .5em 1em;
  font-size: 108%;
  font-weight: bold;
  -moz-border-radius: 10px;
  -webkit-border-radius: 10px;
  -khtml-border-radius: 10px;
  border-radius: 10px;
}

#work img { vertical-align: middle; }

/* =========== Flash ======== */

#flash, #flashAlert {
  padding: 1em 1em 1.1em 55px;
  margin: .5em 0 .5em 0;
  border: 1px solid #ddddcc;
  font-size: 108%;
  font-weight: bold;
}
#flash {
  color: #339933;
  background: #fafafa url(../Images/flash_info.png) no-repeat 5px 5px;
}
#flashAlert {
  color: #993333;
  background: #fafafa url(../Images/flash_alert.png) no-repeat 5px 5px;
}

/* ====== actionParams ====== */

#actionParams {
  margin: 0 -5px 0 -8px;
  padding: 5px .4em 5px .4em;
  background-color: #e8f0f7;
  border-bottom: #999999 dotted 1px;
  text-shadow: none;
}
#actionParams .submit { text-align: right; margin-top: .2em; }
#actionParams .last { margin: .3em 0 0 0; }

td #actionParams { border-bottom: none; padding: .2em 0 .3em 0; margin: 0; }
td #actionParams .formItem { margin-bottom: .1em; }
td #actionParams .formItem label { float: none; width: auto; }
td #actionParams .formItem div { margin: 0; padding: 0;}
td #actionParams .formItem input { width: 90%; }
td #actionParams .formItem input[type="file"] { width: auto; }

/* =========== Vcs ========== */

.vcsUser { font-size: 85%; }
.vcsUser input[type="text"], .vcsUser input[type="password"] {
  width: auto;
}

/* ========== Files ========== */

.files li { margin-bottom: .3em; }

.files #actionParams {
  font-size: 93%;
  margin: .2em 0 .2em 0;
  border-bottom: none;
}

/* =========== Log ========== */

#logLines {
  font-size: 85%;
  color: #4f4f4f;
  margin-top: .5em;
  padding: .8em .5em .8em .5em;
  background-color: #fafafa;
  border: 1px solid #bbbbbb;
}

/* ========= Content ======== */

#content legend {
  color: #004080;
  font-weight: bold;
  font-style: normal;    
  margin-bottom: .5em;
}

#content pre {
  padding: .8em .5em .8em .5em;
  background-color: #fafafa;
  border: 1px solid #bbbbbb;
}

/* ========= Progress ======= */

#progressGauge {
  height: 15px;
  border: 1px solid #bbbbbb;
}
.progressGauge {
  height: 8px;
  border: 1px solid #bbbbbb;
}

#progressGauge div {
  height: 15px;
  background: transparent url(../Images/progress.gif) repeat-x left top;
}
.progressGauge div {
  height: 8px;
  background: transparent url(../Images/progress.gif) repeat-x left top;
}

/* ========== Search ======== */

#search { margin: 10px 0 .4em 0; }

#search input[type="text"] { width: 100%; }
#search img,
#search input[type="image"] { vertical-align: middle; }

#search .submit { text-align: right; }

/* =========== Misc ========= */

.ok { color: #339933; }
.ko { color: #ff0000; }
.error { color: red; font-family: sans-serif; font-size: 85%; font-weight: bold; }
.confirm { color: #3333ff; }
.more a { color: #7f7f7f; font-size: 93%; }
.caption {  color: #7f7f7f; display: block; ; margin: .5em 0 .3em 0; }
.HTTPException { font-size: 167%; color: #993333; margin: 3em; text-align: center; }