/* $Id: login.css f482237c022c 2011/11/03 11:15:44 patrick $ */

/* ======= Presentation ===== */

#mz { clear: both; background-color: white; min-width: 40em; padding: 10px 10px 2em 10px; }
#mz h1 { font-size:240%; color:black; margin:0; padding: 1.5em 1em 1.5em 1em; }
#mz h1 span { color:#993333; }

/* =========== Top ========== */

#top { 
  height:40px;
  background: #993333 url('../Images/breadcrumbs_top.png') repeat-x top left;
}

/* ========== Login ========= */

#loginZone { width: 35em; margin: 1em auto 2em auto; }

#loginTop {
  height: 20px;
  background: transparent url('../Images/login_top_center.png') repeat-x top left;
}
#loginTopLeft {
  float: left;
  width: 16px;
  height: 20px;
  background: transparent url('../Images/login_top_left.png') no-repeat top left;
}
#loginTopRight {
  float: right;
  width: 20px;
  height: 20px;
  background: transparent url('../Images/login_top_right.png') no-repeat top left;
}

#loginMiddle1 {
  background: transparent url('../Images/login_middle_left.png') repeat-y top left;
}
#loginMiddle2 {
  padding: 0 20px 0 16px;
  background: transparent url('../Images/login_middle_right.png') repeat-y top right;
}

#loginBottom {
  height: 20px;
  background: transparent url('../Images/login_bottom_center.png') repeat-x top left;
}
#loginBottomLeft {
  float: left;
  width: 16px;
  height: 20px;
  background: transparent url('../Images/login_bottom_left.png') no-repeat top left;
}
#loginBottomRight {
  float: right;
  width: 20px;
  height: 20px;
  background: transparent url('../Images/login_bottom_right.png') no-repeat top left;
}

#loginForm {
  padding: 6px 14px 14px 8px;
  background-color: #ddddcc;
}

.formItem label { width: 49%; text-align: right; color: black; font-weight:normal; }
.formItem div, .formItem span { margin-left: 50%; text-align: left; }
.formItem input[type="text"], .formItem input[type="password"] { width: 100%; }
#lostPwd { font-size: 85%; font-family: sans-serif; }
#lostPwd a { color: #7f7f7f; }
.submit { text-align: right; }
*+html .formItem input[type="text"],
*+html .formItem input[type="password"] { width: 11.5em; }

/* =========== Flash ======== */

#flashZone {
  width: 24em;
  margin: 1em auto 0 auto;
}

/* ========== Info ========= */

#info {
  width: 38em;
  margin: 3em auto 0 auto;
  padding: 5px;
  background-color: #ffffdd;
  border: 1px dashed #bbbbbb;
}

#info h3 { margin: 0 0 .3em 0; }
