/* $Id: publiquiz.css 264c524fa587 2012/03/22 08:49:09 patrick $ */


/* ====== pquizButton ====== */

.pquizButton {
  cursor: pointer;
  padding: .1em .4em .1em .4em;
  border: 1px solid #999999;
  color: #004080;
  background: #cccccc url('../Images/button.png') repeat-x left top;
  text-decoration: none;
  font-size: 93%;
  font-weight: normal;
}
a.pquizButton, a.pquizButton:visited {
  padding: .15em .45em .15em .45em;
  color: #004080;
}
a.pquizButton img { vertical-align: middle; }
a.pquizButton:hover { text-decoration: none; }
a.pquizButton:hover, input.pquizButton:hover {
  background: #dddddd url('../Images/button_over.png') repeat-x left top;
}

table {clear: left;}

/* ======= TOP LEVEL ====== */


/* ~~~~~~~ pquizQuiz ~~~~~~ */

div.pquizQuiz {
  padding: .8em 0 .8em 0;
  border-bottom: 1px dotted #000000;
}

/* ~~~~~~ pquizSubmit ~~~~~ */

.pquizSubmit {
  margin-top: 1em;
  background-color: #fafafa;
  padding: .5em;
  text-align: right;
}


/* ===== SECTION LEVEL ==== */

/* ~~~ pquizInstructions ~~ */

.pquizInstructions {
  font-weight: bold;
  color: #004080;
}

.pquizInstructions .pdocSection1,
.pquizInstructions .pdocSection2,
.pquizInstructions .pdocSection3  {
  font-family: sans-serif;
}

.pquizInstructions .pdocSection-ex {
  font-weight: normal;
}

/* ~~~~~~ pquizEngine ~~~~~ */

.pquizEngine {
  margin-top: .8em;
}

.pquizEngines {
  margin: .8em 1em 0 1.5em;
}

.pquizEngines li {
  margin-bottom: 1em;
}

/* ~~~~~~~ pquizText ~~~~~~ */

.pquizText {
  line-height: 1.8em;
}

.pquizText td {
  padding: .3em .5% .3em .5%;
}

/* ~~~~~~~ pquizHelp ~~~~~~ */

.pquizHelp {
  margin-top: 1em;
}

/* ~~~~~~ pquizAnswer ~~~~~ */

.pquizAnswer {
  display: none;
  margin-top: 1em;
}

/* ~~~~~~~ pdocMedia ~~~~~~ */

.pquizMatching .pdocMedia {
  margin-top: 0;
}


/* ===== BLOCK LEVEL ====== */

.pquizContent {
  clear: left;
}

/* ~~~~~ pquizChoices ~~~~~ */

.pquizChoices {
  margin: .3em 0 .3em 0;
}

.pquizChoices li {
  list-style: none inside;
  margin-bottom: .2em;
}

.pquizItems {
  padding: 1em 0 0 1em;
  margin-bottom: 1em;
  background-color: #fafafa;
  -moz-border-radius: 10px;
  -webkit-border-radius: 10px;
  -khtml-border-radius: 10px;
  border-radius: 10px;
  border: 1px solid #bbbbbb;
  box-shadow: 2px 2px 3px #bbbbbb;
  line-height: 2em;
  min-height : 3.6em;
  min-width : 8em;
  float:left;
}

.pquizItem {
  float:left;
  padding: .2em 1em .2em 1em;
  margin-right: 1em;
  margin-bottom: 1em;
  background-color: #e8f0f7;
  -moz-border-radius: 5px;
  -webkit-border-radius: 5px;
  -khtml-border-radius: 5px;
  border-radius: 5px;
  border: 1px solid #7f7f7f;
  white-space: nowrap;
}

.pquizItem_dropped{
  float : none;
  margin-right : 0;
}

.pquizMatching td {
  border: none;
  vertical-align: top;
}

.pquizMatching .pquizDrop {
  color: #004080;
  font-weight: bold;
}

.pquizMatching .mode_answer {
    color: #1fbe1f;
}

.point .selected {
  color : #ffffff;
  background-color : #0086cb;
}

.point .mode_answer {
  background-color : #1fbe1f;
}


/* ~~~~ pquizProduction ~~~ */

.pquizProduction {
  margin-top: 1em;
  width: 100%;
  height: 12em;
}


/* ======== INLINES ======= */

span.pquizChoice {
  padding: .2em 0 .2em 0;
  background-color: #e8f0f7;
  -moz-border-radius: 5px;
  -webkit-border-radius: 5px;
  -khtml-border-radius: 5px;
  border-radius: 5px;
}

span.pquizDrop {
  color: #004080;
  font-weight: bold;
}

span.mode_answer {
  color: #1fbe1f;
}

td input.pquizChoice {
  width: 99%;
}

input.disabled {
    opacity: 1;
    background-color: #ffffff;
    color: #000000;
}

input.mode_answer {
    color: #006100;
}

.mode_answer {
    color: #1fbe1f;
}
